#!/bin/env bash

## bash library of test helpers
## version from 2011-21-07

#### HELPER LIST ####
# makeEnvironment {root|user}
# beginTest <name>
# endTest <result> {[true]|false|<errcode>}
# countResult <result> {true|false|<errcode>} <comment> 
# printTestResults
# btllog
# btldetail
#### END OF LIST ####

### WARNING ############################################
## DO NOT USE btllog & btldetail within test progress  #
## pack particular test into function then | btldetail #
########################################################

## usual test process

# makeEnvironment
#   <prepare own things>
# ## basic test ##  
#   beginTest "<name appears in log>"
#     <run stuff>
#     RESULT=<number of how test go, e.g. exit code =$?>
#   endTest ${RESULT} true   ## true expectin errcode=0
#
# ## multi-step test ##
#   beginTest "<name>"
#     <run step 1>
#     result=...
#     countResult ${result} true "<step comment, could be run command, etc.>"
#     ...
#     <run step n>
#     result=...
#     countResult ${result} true "<step comment>"
#  endTest ${LOCALSCORE} true  ## HERE HAVE TO USE ${LOCALSCORE}
#
#  printTestResults
#

####
## specify name of logfiles to backup with timestamp
## $BACKUP_LOG_LIST ## DO NOT UNCOMMENT
####



BASENAME="$(basename $0)"
LOGDIR="logs/"
DETAILFILE="${LOGDIR}$(echo "${BASENAME}" | sed s/.sh$//).detail"
LOGFILE="${LOGDIR}$(echo "${BASENAME}" | sed s/.sh$//).log"


## to detail log commands use { ... } 2>&1 | tee ${DETAILFILE}


## Prepare Testing Environment, and prepare log and detailfile
function makeEnvironment { # 1:{root|user}
  TESTCOUNT=0
  TESTOK=0
  RESULT=0

  cur_time="$(date '+%F %T')"
  cur_time_stamp=$(echo $cur_time | tr ' ' '_')

  if [[ ! -d "${LOGDIR}" ]]; then
    mkdir -p "${LOGDIR}"
  fi

  if [[ ! (-r "${LOGDIR}" && -w "${LOGDIR}") ]]; then
    LOGDIR=
  fi

    ## obtain time stamp from previous log or time stamp current and note
  prev_time=$(grep "TESTLOG" ${LOGFILE} | awk '{print $3 $4};' | tr ' ' '_')
  prev_time=${prev_time:-${cur_time_stamp}-previous}

## backup list of logs
  BACKUP_LOG_LIST+=( ${LOGFILE} ${DETAILFILE} )

  for xlog in ${BACKUP_LOG_LIST[@]} ; do
    if [[ -r ${xlog} ]]; then
      mv ${xlog} "${xlog}.${prev_time}.bak"
    fi
  done

  # timestamp basic test log and detail
  echo " TESTLOG ${BASENAME} ${cur_time}" >> ${LOGFILE}
  echo " TESTDETAIL ${BASENAME} ${cur_time_stamp}" >> ${DETAILFILE}


  if [[ "$1" == "root" && "$(whoami)" != "root" ]]; then
    echo "Please run test as root"
    echo "NON root permission ... aborting" >> ${LOGFILE}
    exit
  fi
}

function beginTest {
  if [[ $# != 1 ]]; then
    echo "TEST ERROR: beginTest parameter"
    exit
  fi
  TESTNAME="$1" 
  TESTCOUNT=$(( ${TESTCOUNT} + 1 ))
  echo " ${TESTCOUNT}: ${TESTNAME}"
  echo -n " ${TESTCOUNT}: ${TESTNAME}" >> ${LOGFILE}
  #echo " ${TESTCOUNT}: ${TESTNAME}" >> ${DETAILFILE}
  LOCALSCORE=0  ## for result counting
  ERRORDETAILLOG=
}

function endTest { ## 1:result 2:expected {true|false|<errcode>}
  if [[ $# -lt 1 ]]; then
    echo "TEST ERROR: endTest parameter"
    exit
  fi


  EXPECT=true
  if [[ $2 == false ]]; then
    EXPECT=false
  elif [[ "$2" =~ ^[0-9][0-9]*$ ]]; then  ## specified errcode
    EXPECT=$2
  fi

## expecting 0 and got it OR not 0 and got other -> OK
## UNIFIED logic as in countResult
  if [[ (${EXPECT} == true && "$1" == "0") \
     || (${EXPECT} == false && "$1" != "0" ) \
     || (${EXPECT} != true && ${EXPECT} != false && $1 == ${EXPECT}) ]]; then
    TESTOK=$(( ${TESTOK} + 1 ))
    echo " w/ ${1} expected ${EXPECT} ... PASS" >> ${LOGFILE}
  else ## did not get expected
    echo " w/ ${1} expected ${EXPECT} ... FAIL" >> ${LOGFILE}
    echo -e "${ERRORDETAILLOG}" >> ${LOGFILE}
  fi
  ERRORDETAILLOG=
}

function printTestResults {
  echo -e "--------------------------\n\
 Passed: ${TESTOK}/${TESTCOUNT} \n\
 Errors: $(( ${TESTCOUNT} - ${TESTOK} )) " >> ${LOGFILE}

  echo -e "\n\n #--------------- SUMMARY ----------------\n\n"
  #cat ${LOGFILE} >> ${DETAILFILE}
  cat ${LOGFILE}
}


## for error count value of 1 instead of real error code
## preparing error explanation w/ real errcode for logging
## to ${ERRORDETAILLOG}, which endTest writes to log and clear
function countResult { #1:result 2:expected {true|false|<errcode>} 3:comment ##runned command, etc.
  if [[ $# -lt 1 ]]; then
    echo "TEST ERROR: countResult parameter"
    exit
  fi

  EXPECT=true
  if [[ $2 == false ]]; then
    EXPECT=false
  elif [[ "$2" =~ ^[0-9][0-9]*$ ]]; then  ## specified errcode
    EXPECT=$2
  fi

## expecting 0 and did not get it OR not 0 and got it OR expeting X and got Y -> FAIL
## UNIFIED logic as in endTest
  if [[ (${EXPECT} == true && $1 == 0) \
     || (${EXPECT} == false && $1 != 0) \
     || (${EXPECT} != true && ${EXPECT} != false && $1 == ${EXPECT}) ]]; then
     : #noop
  else
    LOCALSCORE=$(( ${LOCALSCORE} + 1 ))
    ERRORDETAILLOG="${ERRORDETAILLOG} \t* ${3}: exited with $1 expected ${EXPECT}\n"
  fi
}


## reads stdin and loggin into DETAIL FILE 
## usual use "myTest.sh | btldetail"
## DO NOT use it within test progress, causes wrong exitcode !!!
function btldetail {
  tee -a ${DETAILFILE}
}

## reads stdin and loggin into DETAIL FILE 
## usual use "myTestFunction | btldetail"
## DO NOT use it within test progress, causes wrong exitcode !!!
function btllog {
  tee -a ${LOGFILE}
}

